@@ -11,7 +11,6 @@ import re
 import sys
 import warnings
 
-from typing import List, Tuple
 
 import pytest
 
@@ -23,17 +22,12 @@ from coverage.types import TArc
 from tests.coveragetest import CoverageTest
 from tests.helpers import (
     CheckUniqueFilenames, FailingProxy,
-    arcs_to_arcz_repr, arcz_to_arcs, assert_count_equal, assert_coverage_warnings,
+    all_our_source_files,
+    arcz_to_arcs, assert_count_equal, assert_coverage_warnings,
     re_lines, re_lines_text, re_line,
 )
 
 
-def test_xdist_sys_path_nuttiness_is_fixed() -> None:
-    # See conftest.py:fix_xdist_sys_path
-    assert sys.path[1] != ""
-    assert os.getenv("PYTHONPATH") is None
-
-
 def test_assert_count_equal() -> None:
     assert_count_equal(set(), set())
     assert_count_equal({"a": 1, "b": 2}, ["b", "a"])
@@ -67,23 +61,23 @@ class CoverageTestTest(CoverageTest):
         self.assert_file_count("*.q", 0)
         msg = re.escape(
             "There should be 13 files matching 'a*.txt', but there are these: " +
-            "['abcde.txt', 'afile.txt', 'axczz.txt']"
+            "['abcde.txt', 'afile.txt', 'axczz.txt']",
         )
         with pytest.raises(AssertionError, match=msg):
             self.assert_file_count("a*.txt", 13)
         msg = re.escape(
             "There should be 12 files matching '*c*.txt', but there are these: " +
-            "['abcde.txt', 'axczz.txt']"
+            "['abcde.txt', 'axczz.txt']",
         )
         with pytest.raises(AssertionError, match=msg):
             self.assert_file_count("*c*.txt", 12)
         msg = re.escape(
-            "There should be 11 files matching 'afile.*', but there are these: ['afile.txt']"
+            "There should be 11 files matching 'afile.*', but there are these: ['afile.txt']",
         )
         with pytest.raises(AssertionError, match=msg):
             self.assert_file_count("afile.*", 11)
         msg = re.escape(
-            "There should be 10 files matching '*.q', but there are these: []"
+            "There should be 10 files matching '*.q', but there are these: []",
         )
         with pytest.raises(AssertionError, match=msg):
             self.assert_file_count("*.q", 10)
@@ -233,7 +227,7 @@ class CheckUniqueFilenamesTest(CoverageTest):
             filename: str,
             a: int = 17,
             b: str = "hello",
-        ) -> Tuple[int, str, int, str]:
+        ) -> tuple[int, str, int, str]:
             """The method we'll wrap, with args to be sure args work."""
             return (self.x, filename, a, b)
 
@@ -266,38 +260,34 @@ class CheckCoverageTest(CoverageTest):
             b = 10
         assert a == 6 and b == 10
         """
-    ARCZ = ".1 12 -23 34 3-2 4-2 25 56 67 78 8B 9A AB B."
-    ARCZ_MISSING = "3-2 78 8B"
-    ARCZ_UNPREDICTED = "79"
+    BRANCHZ = "34 3-2"
+    BRANCHZ_MISSING = "3-2"
 
-    def test_check_coverage_possible(self) -> None:
-        msg = r"(?s)Possible arcs differ: .*- \(6, 3\).*\+ \(6, 7\)"
-        with pytest.raises(AssertionError, match=msg):
+    def test_check_coverage_possible_branches(self) -> None:
+        msg = "Wrong possible branches: [(7, -2), (7, 4)] != [(3, -2), (3, 4)]"
+        with pytest.raises(AssertionError, match=re.escape(msg)):
             self.check_coverage(
                 self.CODE,
-                arcz=self.ARCZ.replace("7", "3"),
-                arcz_missing=self.ARCZ_MISSING,
-                arcz_unpredicted=self.ARCZ_UNPREDICTED,
+                branchz=self.BRANCHZ.replace("3", "7"),
+                branchz_missing=self.BRANCHZ_MISSING,
             )
 
-    def test_check_coverage_missing(self) -> None:
-        msg = r"(?s)Missing arcs differ: .*- \(3, 8\).*\+ \(7, 8\)"
-        with pytest.raises(AssertionError, match=msg):
+    def test_check_coverage_missing_branches(self) -> None:
+        msg = "Wrong missing branches: [(3, 4)] != [(3, -2)]"
+        with pytest.raises(AssertionError, match=re.escape(msg)):
             self.check_coverage(
                 self.CODE,
-                arcz=self.ARCZ,
-                arcz_missing=self.ARCZ_MISSING.replace("7", "3"),
-                arcz_unpredicted=self.ARCZ_UNPREDICTED,
+                branchz=self.BRANCHZ,
+                branchz_missing="34",
             )
 
-    def test_check_coverage_unpredicted(self) -> None:
-        msg = r"(?s)Unpredicted arcs differ: .*- \(3, 9\).*\+ \(7, 9\)"
-        with pytest.raises(AssertionError, match=msg):
+    def test_check_coverage_mismatched_missing_branches(self) -> None:
+        msg = "branches_missing = [(1, 2)], has non-branches in it."
+        with pytest.raises(AssertionError, match=re.escape(msg)):
             self.check_coverage(
                 self.CODE,
-                arcz=self.ARCZ,
-                arcz_missing=self.ARCZ_MISSING,
-                arcz_unpredicted=self.ARCZ_UNPREDICTED.replace("7", "3")
+                branchz=self.BRANCHZ,
+                branchz_missing="12",
             )
 
 
@@ -376,26 +366,9 @@ class ArczTest(CoverageTest):
         ("-11 12 2-5", [(-1, 1), (1, 2), (2, -5)]),
         ("-QA CB IT Z-A", [(-26, 10), (12, 11), (18, 29), (35, -10)]),
     ])
-    def test_arcz_to_arcs(self, arcz: str, arcs: List[TArc]) -> None:
+    def test_arcz_to_arcs(self, arcz: str, arcs: list[TArc]) -> None:
         assert arcz_to_arcs(arcz) == arcs
 
-    @pytest.mark.parametrize("arcs, arcz_repr", [
-        ([(-1, 1), (1, 2), (2, -1)], "(-1, 1) # .1\n(1, 2) # 12\n(2, -1) # 2.\n"),
-        ([(-1, 1), (1, 2), (2, -5)], "(-1, 1) # .1\n(1, 2) # 12\n(2, -5) # 2-5\n"),
-        ([(-26, 10), (12, 11), (18, 29), (35, -10), (1, 33), (100, 7)],
-            (
-            "(-26, 10) # -QA\n" +
-            "(12, 11) # CB\n" +
-            "(18, 29) # IT\n" +
-            "(35, -10) # Z-A\n" +
-            "(1, 33) # 1X\n" +
-            "(100, 7) # ?7\n"
-            )
-        ),
-    ])
-    def test_arcs_to_arcz_repr(self, arcs: List[TArc], arcz_repr: str) -> None:
-        assert arcs_to_arcz_repr(arcs) == arcz_repr
-
 
 class AssertCoverageWarningsTest(CoverageTest):
     """Tests of assert_coverage_warnings"""
@@ -472,3 +445,13 @@ def test_failing_proxy() -> None:
         proxy.add(3, 4)
     # then add starts working
     assert proxy.add(5, 6) == 11
+
+
+def test_all_our_source_files() -> None:
+    # Twas brillig and the slithy toves
+    i = 0
+    for i, (source_file, source) in enumerate(all_our_source_files(), start=1):
+        has_toves = (source_file.name == "test_testing.py")
+        assert (("# Twas brillig " + "and the slithy toves") in source) == has_toves
+        assert len(source) > 190    # tests/__init__.py is shortest at 196
+    assert 120 < i < 200            # currently 125 files
